use axum::response::sse;
use serde::Serialize;
use tokio::sync::broadcast::{self, Receiver, Sender};
use ts_rs::TS;
use utoipa::ToSchema;

use crate::api::routes::{units::models::Unit, vendors::models::Vendor};

#[derive(Debug, Clone, Serialize, TS, ToSchema)]
#[ts(export)]
pub struct DataHubStream {
    /// Unique identifier for the data update event
    id: String,
    /// Event Type
    event: EventType,
    /// Data content
    data: String,
}

#[derive(Debug, Clone, Serialize, TS, ToSchema, strum::Display)]
#[ts(export)]
pub enum EventType {
    Units,
    Vendors,
}

#[derive(Debug)]
pub struct DataHub {
    pub sender: Sender<DataHubData>,
    pub receiver: Receiver<DataHubData>,
}

impl DataHub {
    pub fn new() -> Self {
        let (sender, receiver) = broadcast::channel(15);

        Self {
            sender,
            receiver: receiver,
        }
    }
}

#[derive(Debug, Clone, Serialize)]
pub enum DataHubData {
    Units(Vec<Unit>),
    Vendors(Vec<Vendor>),
}

impl From<DataHubData> for sse::Event {
    fn from(value: DataHubData) -> Self {
        let (event_type, data) = match value {
            DataHubData::Units(units) => (
                EventType::Units,
                serde_json::to_string(&units).unwrap_or_default(),
            ),
            DataHubData::Vendors(vendors) => (
                EventType::Vendors,
                serde_json::to_string(&vendors).unwrap_or_default(),
            ),
        };

        sse::Event::default()
            .event(event_type.to_string())
            .data(data)
    }
}
